#
# Extract the names of nodes from the Ruby header node.h, looking
# for the typedef.
#


require 'mkmf'
require "rbconfig"

# Return the full path to node.h

def nameOfNode_h
  cfg = Config::CONFIG
  srcdir = cfg["srcdir"]
  libdir = cfg["libdir"]+"/ruby/"+cfg["MAJOR"]+"."+cfg["MINOR"]
  archdir = libdir+"/"+cfg["arch"]
  
  if File.exist? archdir + "/ruby.h"
    hdrdir = archdir
  else File.exist? srcdir + "/ruby.h"
    hdrdir = srcdir
  end
  
  hdrdir + "/node.h"
end

# Given a list of node names, check that our nodeDump.c file
# handles them all

# these are internal Ruby nodes that don't appear in a parse tree

DUMMY_NODES = %w( NODE_MEMO NODE_LAST NODE_ALLOCA NODE_CNAME
                  NODE_CFUNC NODE_IFUNC NODE_RESBODY)


def checkWeHandle(nameList)
  nodes = File.new(File.join(File.dirname($0), "nodeDump.c")).grep(/case\s+(NODE_\w+):/) { $1 }
  inRubyNotDump = nameList - nodes - DUMMY_NODES
  inDumpNotRuby = nodes - nameList

  if !inRubyNotDump.empty?
    $stderr.print "\n\nNodes we don't handle:\n", inRubyNotDump.join(", "), "\n"
  end

  if !inDumpNotRuby.empty?
    $stderr.print "\n\nNodes we handle not in Ruby:\n", inDumpNotRuby.join(", "), "\n"
  end

  # fail if there's anything in Ruby that we don't now about
  return inRubyNotDump.empty?
end


# Create a node_names.h file containing enum -> name mappings

def createNodeNames
  
  nameList = []

  node = nameOfNode_h()

  File.open("node_names.h", "w") do |op|
    
    copying = false
    
    op.puts "const char *node_names[] = {"
    
    IO.foreach(node) do |line|
      
      if copying
        if line =~ /\};/
          copying = false
        else
          if line =~ /^#/
            op.puts line
          else
            if line =~ /\s*(\w+)(,)?/
              op.puts "\t\"#$1\"#$2"
              op.puts "\#define\t #{$1} #{$1}"
              nameList << $1
            end
          end
        end
      end
        
      copying = true  if line =~ /enum node_type/
    end
      
    op.puts "};"
  end

  checkWeHandle(nameList)
end

if createNodeNames()

  have_func("rb_proc_new", "ruby.h")

  $CPPFLAGS << " -I."
  create_makefile("NodeDump")

else

  $stderr.puts <<_EOM_

=================================================================

NodeDump can not be built on this system. This is probably due to
an incompatibility with Ruby versions. This version of NodeDump
only runs with Ruby 1.6 or later.

=================================================================

_EOM_

end
